@@ -1,3 +1,34 @@
+function memory_consumed_str(size_in_mb) {
+  // Return a string corresponding to amount of memory consumed.
+  let gigabytes = Math.floor(size_in_mb / 1024);
+  let terabytes = Math.floor(gigabytes / 1024);
+  if (terabytes > 0) {
+    return `${(size_in_mb / 1048576).toFixed(3)} TB`;
+  } else if (gigabytes > 0) {
+    return `${(size_in_mb / 1024).toFixed(3)} GB`;
+  } else {
+    return `${size_in_mb.toFixed(3)} MB`;
+  }
+}
+
+function time_consumed_str(time_in_ms) {
+  let hours = Math.floor(time_in_ms / 3600000);
+  let minutes = Math.floor((time_in_ms % 3600000) / 60000);
+  let seconds = Math.floor((time_in_ms % 60000) / 1000);
+  let hours_exact = time_in_ms / 3600000;
+  let minutes_exact = (time_in_ms % 3600000) / 60000;
+  let seconds_exact = (time_in_ms % 60000) / 1000;
+  if (hours > 0) {
+    return `${hours_exact.toFixed(0)}h:${minutes_exact.toFixed(0)}m:${seconds_exact.toFixed(3)}s`;
+  } else if (minutes > 0) {
+    return `${minutes_exact.toFixed(0)}m:${seconds_exact.toFixed(3)}s`;
+  } else if (seconds > 0) {
+    return `${seconds_exact.toFixed(3)}s`;
+  } else {
+    return `${time_in_ms.toFixed(3)}ms`;
+  }
+}
+
 function makeBar(python, native, system) {
   return {
     $schema: "https://vega.github.io/schema/vega-lite/v5.json",
@@ -174,12 +205,12 @@ function makeMemoryBar(memory, title, python_percent, total, color) {
         {
           x: 0,
           y: python_percent * memory,
-          c: "(Python) " + (python_percent * memory).toFixed(1) + "MB",
+          c: "(Python) " + memory_consumed_str(python_percent * memory),
         },
         {
           x: 0,
           y: (1.0 - python_percent) * memory,
-          c: "(native) " + ((1.0 - python_percent) * memory).toFixed(1) + "MB",
+          c: "(native) " + memory_consumed_str((1.0 - python_percent) * memory),
         },
       ],
     },
@@ -213,13 +244,13 @@ function makeSparkline(
   const values = samples.map((v, i) => {
     let leak_str = "";
     if (leak_velocity != 0) {
-      leak_str = `; possible leak (${leak_velocity.toFixed(1)} MB/s)`;
+	leak_str = `; possible leak (${memory_consumed_str(leak_velocity)}/s)`;
     }
     return {
       x: v[0],
       y: v[1],
       y_text:
-        v[1].toFixed(1) + "MB (@ " + (v[0] / 1e9).toFixed(0) + "s)" + leak_str,
+        memory_consumed_str(v[1]) + " (@ " + time_consumed_str(v[0] / 1e6) + ")" + leak_str,
     };
   });
   let leak_info = "";
@@ -361,10 +392,10 @@ function makeTableHeader(fname, gpu, memory, functions = false) {
         info: "% of bytes allocated by line / function over total bytes allocated in file",
       },
       {
-        title: ["copy", "(MB/s)"],
+        title: ["copy", ""],
         color: CopyColor,
         width: 0,
-        info: "Rate of copying memory, in megabytes per second",
+        info: "Rate of copying memory",
       },
     ]);
   }
@@ -573,9 +604,7 @@ async function display(prof) {
     s += `<td><font style="font-size: small"><b>Memory:</b> <font color="darkgreen">Python</font> | <font color="#50C878">native</font><br /></font></td>`;
     s += '<td width="10"></td>';
     s += '<td valign="middle" style="vertical-align: middle">';
-    s += `<font style="font-size: small"><b>Memory timeline: </b>(max: ${prof.max_footprint_mb.toFixed(
-      1
-    )}MB, growth: ${prof.growth_rate.toFixed(1)}%)</font>`;
+      s += `<font style="font-size: small"><b>Memory timeline: </b>(max: ${memory_consumed_str(prof.max_footprint_mb)}, growth: ${prof.growth_rate.toFixed(1)}%)</font>`;
     s += "</td>";
   }
   s += "</tr>";
@@ -650,7 +679,7 @@ async function display(prof) {
   s += '<div class="container-fluid">';
 
   // Convert files to an array and sort it in descending order by percent of CPU time.
-  files = Object.entries(prof.files);
+  let files = Object.entries(prof.files);
   files.sort((x, y) => {
     return y[1].percent_cpu_time - x[1].percent_cpu_time;
   });
@@ -662,7 +691,7 @@ async function display(prof) {
       ff[0]
     }</code>: % of time = ${ff[1].percent_cpu_time.toFixed(
       1
-    )}% out of ${prof.elapsed_time_sec.toFixed(1)}s.</font></p>`;
+    )}% (${time_consumed_str(ff[1].percent_cpu_time / 100.0 * prof.elapsed_time_sec * 1e3)}) out of ${time_consumed_str(prof.elapsed_time_sec * 1e3)}.</font></p>`;
     s += "<div>";
     s += `<table class="profile table table-hover table-condensed" id="table-${tableID}">`;
     tableID++;
