@@ -3,6 +3,9 @@ import linecache
 import os
 import random
 import re
+import statistics
+import sys
+
 from collections import OrderedDict, defaultdict
 from operator import itemgetter
 from pathlib import Path
@@ -11,8 +14,39 @@ from typing import Any, Callable, Dict, List
 from scalene.scalene_leak_analysis import ScaleneLeakAnalysis
 from scalene.scalene_statistics import Filename, LineNumber, ScaleneStatistics
 
+if sys.platform != "win32":
+    from scalene.crdp import rdp
+
 
 class ScaleneJSON:
+    @staticmethod
+    def memory_consumed_str(size_in_mb: float) -> str:
+        """Return a string corresponding to amount of memory consumed."""
+        gigabytes = size_in_mb // 1024
+        terabytes = gigabytes // 1024
+        if terabytes > 0:
+            return f"{(size_in_mb / 1048576):3.3f} TB"
+        elif gigabytes > 0:
+            return f"{(size_in_mb / 1024):3.3f} GB"
+        else:
+            return f"{size_in_mb:3.3f} MB"
+
+    @staticmethod
+    def time_consumed_str(time_in_ms: float) -> str:
+        hours = time_in_ms // 3600000
+        minutes = (time_in_ms % 3600000) // 60000
+        seconds = (time_in_ms % 60000) // 1000
+        hours_exact = time_in_ms / 3600000
+        minutes_exact = (time_in_ms % 3600000) / 60000
+        seconds_exact = (time_in_ms % 60000) / 1000
+        if hours > 0:
+            return f"{hours_exact:.0f}h:{minutes_exact:.0f}m:{seconds_exact:3.3f}s"
+        elif minutes > 0:
+            return f"{minutes_exact:.0f}m:{seconds_exact:3.3f}s"
+        elif seconds > 0:
+            return f"{seconds_exact:3.3f}s"
+        else:
+            return f"{time_in_ms:3.3f}ms"
 
     # Default threshold for percent of CPU time to report a file.
     cpu_percent_threshold = 1
@@ -56,10 +90,22 @@ class ScaleneJSON:
                 last_mem = mem
 
         if len(samples) > self.max_sparkline_samples:
-            # Too many samples. We randomly downsample.
-            samples = sorted(
-                random.sample(samples, self.max_sparkline_samples)
-            )
+            # Try to reduce the number of samples with the
+            # Ramer-Douglas-Peucker algorithm, which attempts to
+            # preserve the shape of the graph. If that fails to bring
+            # the number of samples below our maximum, randomly
+            # downsample (epsilon calculation from
+            # https://stackoverflow.com/questions/57052434/can-i-guess-the-appropriate-epsilon-for-rdp-ramer-douglas-peucker)
+            epsilon = (len(samples) / (3 * self.max_sparkline_samples)) * 2
+            # print("BEFORE len = ", len(samples))
+            new_samples = rdp(samples, epsilon=epsilon)
+            # print("AFTER len = ", len(new_samples))
+            if len(new_samples) > self.max_sparkline_samples:
+                # We still didn't get enough compression; randomly downsample.
+                new_samples = sorted(
+                    random.sample(new_samples, self.max_sparkline_samples)
+                )
+            samples = new_samples
 
         return samples
 
@@ -78,7 +124,7 @@ class ScaleneJSON:
         """Print at most one line of the profile (true == printed one)."""
 
         full_fname = os.path.abspath(fname)
-        
+
         if not force_print and not profile_this_code(fname, line_no):
             return {}
         # Prepare output values.
@@ -154,7 +200,9 @@ class ScaleneJSON:
         else:
             n_copy_mb_s = 0
 
-        samples = self.compress_samples(
+        stats.per_line_footprint_samples[fname][
+            line_no
+        ] = self.compress_samples(
             stats.per_line_footprint_samples[fname][line_no],
             stats.max_footprint,
         )
@@ -176,7 +224,7 @@ class ScaleneJSON:
             "n_usage_fraction": n_usage_fraction,
             "n_python_fraction": n_python_fraction,
             "n_copy_mb_s": n_copy_mb_s,
-            "memory_samples": samples,
+            "memory_samples": stats.per_line_footprint_samples[fname][line_no],
         }
 
     def output_profiles(
@@ -215,7 +263,7 @@ class ScaleneJSON:
             return {}
         growth_rate = 0.0
         if profile_memory:
-            samples = self.compress_samples(
+            stats.memory_footprint_samples = self.compress_samples(
                 stats.memory_footprint_samples, stats.max_footprint
             )
 
@@ -227,10 +275,11 @@ class ScaleneJSON:
                     / stats.allocation_velocity[1]
                 )
         else:
-            samples = []
+            stats.memory_footprint_samples = []
 
         output: Dict[str, Any] = {
             "program": program,
+            "alloc_samples": stats.alloc_samples,
             "elapsed_time_sec": stats.elapsed_time,
             "growth_rate": growth_rate,
             "max_footprint_mb": stats.max_footprint,
@@ -243,7 +292,7 @@ class ScaleneJSON:
             "files": {},
             "gpu": self.gpu,
             "memory": profile_memory,
-            "samples": samples,
+            "samples": stats.memory_footprint_samples,
         }
 
         # Build a list of files we will actually report on.
@@ -285,7 +334,7 @@ class ScaleneJSON:
             # restore its name, as in "[12]".
             fname_print = fname
 
-            result = re.match("ipython-input-([0-9]+)-.*", fname_print)
+            result = re.match(r"ipython-input-([0-9]+)-.*", fname_print)
             if result:
                 fname_print = Filename("[" + result.group(1) + "]")
 
@@ -335,7 +384,7 @@ class ScaleneJSON:
             try:
                 with open(full_fname, "r", encoding="utf-8") as source_file:
                     code_lines = source_file.readlines()
-            except FileNotFoundError:
+            except (FileNotFoundError, OSError):
                 continue
 
             output["files"][fname_print] = {
@@ -363,7 +412,6 @@ class ScaleneJSON:
                             profile_line
                         )
 
-
             fn_stats = stats.build_function_stats(fname)
             # Check CPU samples and memory samples.
             print_fn_summary = False
