@@ -1,5 +1,6 @@
 import argparse
 import contextlib
+import platform
 import sys
 from textwrap import dedent
 from typing import Any, List, NoReturn, Optional, Tuple
@@ -7,6 +8,8 @@ from typing import Any, List, NoReturn, Optional, Tuple
 from scalene.scalene_arguments import ScaleneArguments
 from scalene.scalene_version import scalene_version, scalene_date
 
+scalene_gui_url = "https://plasma-umass.org/scalene-gui/"
+
 
 class RichArgParser(argparse.ArgumentParser):
     def __init__(self, *args: Any, **kwargs: Any):
@@ -49,9 +52,9 @@ class ScaleneParseArgs:
 
 
 command-line:
-  % [b]scalene \[options] yourprogram.py[/b]
+  % [b]scalene \[options] your_program.py \[--- --your_program_args] [/b]
 or
-  % [b]python3 -m scalene \[options] yourprogram.py[/b]
+  % [b]python3 -m scalene \[options] your_program.py \[--- --your_program_args] [/b]
 
 in Jupyter, line mode:
 [b]  %scrun \[options] statement[/b]
@@ -109,7 +112,7 @@ for the process ID that Scalene reports. For example:
             const=True,
             default=defaults.html,
             help="output as HTML (default: [blue]"
-            + str("html" if defaults.html else "text")
+            + str("html" if defaults.html else "web")
             + "[/blue])",
         )
         parser.add_argument(
@@ -119,7 +122,7 @@ for the process ID that Scalene reports. For example:
             const=True,
             default=defaults.json,
             help="output as JSON (default: [blue]"
-            + str("json" if defaults.json else "text")
+            + str("json" if defaults.json else "web")
             + "[/blue])",
         )
         parser.add_argument(
@@ -136,14 +139,15 @@ for the process ID that Scalene reports. For example:
             action="store_const",
             const=True,
             default=defaults.web,
-            help="writes 'profile.json' and opens the web UI (http://plasma-umass.org/scalene-gui/)",
+            help="opens a web tab to view the profile (saved as 'profile.html')",
         )
         parser.add_argument(
-            "--port",
-            dest="port",
-            type=int,
-            default=defaults.port,
-            help=f"binds the web UI server to this port (default: {defaults.port})",
+            "--viewer",
+            dest="viewer",
+            action="store_const",
+            const=True,
+            default=False,
+            help=f"only opens the web UI ({scalene_gui_url})",
         )
         parser.add_argument(
             "--reduced-profile",
@@ -160,18 +164,40 @@ for the process ID that Scalene reports. For example:
             help=f"output profiles every so many seconds (default: [blue]{defaults.profile_interval}[/blue])",
         )
         parser.add_argument(
-            "--cpu-only",
-            dest="cpu_only",
+            "--cpu",
+            dest="cpu",
             action="store_const",
             const=True,
-            default=defaults.cpu_only,
-            help="only profile CPU+GPU time (default: [blue]profile "
-            + (
-                "CPU only"
-                if defaults.cpu_only
-                else "CPU+GPU, memory, and copying"
-            )
-            + "[/blue])",
+            default=None,
+            help="profile CPU time (default: [blue] True [/blue])",
+        )
+        parser.add_argument(
+            "--cpu-only",
+            dest="cpu",
+            action="store_const",
+            const=True,
+            default=None,
+            help="profile CPU time ([red]deprecated: use --cpu [/red])",
+        )
+        parser.add_argument(
+            "--gpu",
+            dest="gpu",
+            action="store_const",
+            const=True,
+            default=None,
+            help="profile GPU time and memory (default: [blue]"
+            + (str(defaults.gpu))
+            + " [/blue])",
+        )
+        parser.add_argument(
+            "--memory",
+            dest="memory",
+            action="store_const",
+            const=True,
+            default=None,
+            help="profile memory (default: [blue]"
+            + (str(defaults.memory))
+            + " [/blue])",
         )
         parser.add_argument(
             "--profile-all",
@@ -224,7 +250,7 @@ for the process ID that Scalene reports. For example:
         parser.add_argument(
             "--cpu-percent-threshold",
             dest="cpu_percent_threshold",
-            type=int,
+            type=float,
             default=defaults.cpu_percent_threshold,
             help=f"only report profiles with at least this percent of CPU time (default: [blue]{defaults.cpu_percent_threshold}%%[/blue])",
         )
@@ -294,8 +320,36 @@ for the process ID that Scalene reports. For example:
         left += args.unused_args
         import re
 
+        # Launch the UI if `--viewer` was selected.
+        if args.viewer:
+            import webbrowser
+
+            if (
+                webbrowser.get()
+                and type(webbrowser.get()).__name__ != "GenericBrowser"
+            ):
+                webbrowser.open(scalene_gui_url)
+            else:
+                print(f"Scalene: could not open {scalene_gui_url}.")
+            sys.exit(0)
+
+        # If any of the individual profiling metrics were specified,
+        # disable the unspecified ones (set as None).
+        if args.cpu or args.gpu or args.memory:
+            if not args.memory:
+                args.memory = False
+            if not args.gpu:
+                args.gpu = False
+        else:
+            # Nothing specified; use defaults.
+            args.cpu = defaults.cpu
+            args.gpu = defaults.gpu
+            args.memory = defaults.memory
+
+        args.cpu = True  # Always true
+
         in_jupyter_notebook = len(sys.argv) >= 1 and re.match(
-            "ipython-input-([0-9]+)-.*", sys.argv[0]
+            r"ipython-input-([0-9]+)-.*", sys.argv[0]
         )
         # If the user did not enter any commands (just `scalene` or `python3 -m scalene`),
         # print the usage information and bail.
