@@ -19,7 +19,7 @@ import builtins
 import contextlib
 import functools
 import gc
-import http.server
+import importlib
 import inspect
 import json
 import math
@@ -38,8 +38,22 @@ import time
 import traceback
 import webbrowser
 from collections import defaultdict
-from types import FrameType
-from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union, cast
+from importlib.abc import SourceLoader
+from importlib.machinery import ModuleSpec
+from jinja2 import Environment, FileSystemLoader
+from types import CodeType, FrameType
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
 
 from scalene.scalene_arguments import ScaleneArguments
 from scalene.scalene_client_timer import ScaleneClientTimer
@@ -114,6 +128,82 @@ def stop() -> None:
     Scalene.stop()
 
 
+def _get_module_details(
+    mod_name: str,
+    error: Type[Exception] = ImportError,
+) -> Tuple[str, ModuleSpec, CodeType]:
+    """Copy of `runpy._get_module_details`, but not private."""
+    if mod_name.startswith("."):
+        raise error("Relative module names not supported")
+    pkg_name, _, _ = mod_name.rpartition(".")
+    if pkg_name:
+        # Try importing the parent to avoid catching initialization errors
+        try:
+            __import__(pkg_name)
+        except ImportError as e:
+            # If the parent or higher ancestor package is missing, let the
+            # error be raised by find_spec() below and then be caught. But do
+            # not allow other errors to be caught.
+            if e.name is None or (
+                e.name != pkg_name and not pkg_name.startswith(e.name + ".")
+            ):
+                raise
+        # Warn if the module has already been imported under its normal name
+        existing = sys.modules.get(mod_name)
+        if existing is not None and not hasattr(existing, "__path__"):
+            from warnings import warn
+
+            msg = (
+                "{mod_name!r} found in sys.modules after import of "
+                "package {pkg_name!r}, but prior to execution of "
+                "{mod_name!r}; this may result in unpredictable "
+                "behaviour".format(mod_name=mod_name, pkg_name=pkg_name)
+            )
+            warn(RuntimeWarning(msg))
+
+    try:
+        spec = importlib.util.find_spec(mod_name)
+    except (ImportError, AttributeError, TypeError, ValueError) as ex:
+        # This hack fixes an impedance mismatch between pkgutil and
+        # importlib, where the latter raises other errors for cases where
+        # pkgutil previously raised ImportError
+        msg = "Error while finding module specification for {!r} ({}: {})"
+        if mod_name.endswith(".py"):
+            msg += (
+                f". Try using '{mod_name[:-3]}' instead of "
+                f"'{mod_name}' as the module name."
+            )
+        raise error(msg.format(mod_name, type(ex).__name__, ex)) from ex
+    if spec is None:
+        raise error("No module named %s" % mod_name)
+    if spec.submodule_search_locations is not None:
+        if mod_name == "__main__" or mod_name.endswith(".__main__"):
+            raise error("Cannot use package as __main__ module")
+        try:
+            pkg_main_name = mod_name + ".__main__"
+            return _get_module_details(pkg_main_name, error)
+        except error as e:
+            if mod_name not in sys.modules:
+                raise  # No module loaded; being a package is irrelevant
+            raise error(
+                ("%s; %r is a package and cannot " + "be directly executed")
+                % (e, mod_name)
+            )
+    loader = spec.loader
+    # use isinstance instead of `is None` to placate mypy
+    if not isinstance(loader, SourceLoader):
+        raise error(
+            "%r is a namespace package and cannot be executed" % mod_name
+        )
+    try:
+        code = loader.get_code(mod_name)
+    except ImportError as e:
+        raise error(format(e)) from e
+    if code is None:
+        raise error("No code object available for %s" % mod_name)
+    return mod_name, spec, code
+
+
 class Scalene:
     """The Scalene profiler itself."""
 
@@ -125,12 +215,11 @@ class Scalene:
     # the pid of the primary profiler
     __parent_pid = -1
     __initialized: bool = False
-    __last_profiled = (Filename("NADA"), LineNumber(0), ByteCodeIndex(0))
+    __last_profiled = [Filename("NADA"), LineNumber(0), ByteCodeIndex(0)]
     __last_profiled_invalidated = False
     __gui_dir = "scalene-gui"
     __profile_filename = "profile.json"
-    __localhost = "http://localhost"
-    __profiler_html = "profiler.html"
+    __profiler_html = "profile.html"
     __error_message = "Error in program being profiled"
     BYTES_PER_MB = 1024 * 1024
 
@@ -293,7 +382,9 @@ class Scalene:
         bytearray(NEWLINE_TRIGGER_LENGTH)
 
     @staticmethod
-    def invalidate_lines(frame: FrameType, _event: str, _arg: str) -> Any:
+    def invalidate_lines_python(
+        frame: FrameType, _event: str, _arg: str
+    ) -> Any:
         """Mark the last_profiled information as invalid as soon as we execute a different line of code."""
         try:
             # If we are still on the same line, return.
@@ -301,13 +392,13 @@ class Scalene:
             fl = frame.f_lineno
             (fname, lineno, lasti) = Scalene.__last_profiled
             if (ff == fname) and (fl == lineno):
-                return None
+                return Scalene.invalidate_lines_python
             # Different line: stop tracing this frame.
             frame.f_trace = None
             frame.f_trace_lines = False
             # If we are not in a file we should be tracing, return.
-            if not Scalene.should_trace(ff):
-                return None
+            # if not Scalene.should_trace(ff):
+            #     return None
             if f := Scalene.on_stack(frame, fname, lineno):
                 # We are still on the same line, but somewhere up the stack
                 # (since we returned when it was the same line in this
@@ -322,14 +413,14 @@ class Scalene:
                 Scalene.update_line()
             Scalene.__last_profiled_invalidated = True
 
-            Scalene.__last_profiled = (
+            Scalene.__last_profiled = [
                 Filename("NADA"),
                 LineNumber(0),
                 ByteCodeIndex(0)
                 #     Filename(ff),
                 #     LineNumber(fl),
                 #     ByteCodeIndex(frame.f_lasti),
-            )
+            ]
             return None
         except AttributeError:
             # This can happen when Scalene shuts down.
@@ -464,6 +555,11 @@ class Scalene:
         this_frame: Optional[FrameType],
     ) -> None:
         """Handle allocation signals."""
+        if not Scalene.__args.memory:
+            # This should never happen, but we fail gracefully.
+            return
+        from scalene import pywhere
+
         if this_frame:
             Scalene.enter_function_meta(this_frame, Scalene.__stats)
         # Walk the stack till we find a line of code in a file we are tracing.
@@ -482,26 +578,30 @@ class Scalene:
         # First, see if we have now executed a different line of code.
         # If so, increment.
         # TODO: assess the necessity of the following block
-        invalidated = Scalene.__last_profiled_invalidated
+        invalidated = pywhere.get_last_profiled_invalidated()
         (fname, lineno, lasti) = Scalene.__last_profiled
+        # Requesting review on this, I think this is semantically more correct
+        # but I'm not fully sure
         if not invalidated and not (
-            fname == Filename(f.f_code.co_filename)
-            and lineno == LineNumber(f.f_lineno)
+            Scalene.on_stack(this_frame, fname, lineno)
+            # fname == Filename(f.f_code.co_filename)
+            # and lineno == LineNumber(f.f_lineno)
         ):
+
             with Scalene.__invalidate_mutex:
-                Scalene.__invalidate_queue.append((fname, lineno))
+                Scalene.__invalidate_queue.append(
+                    (Scalene.__last_profiled[0], Scalene.__last_profiled[1])
+                )
                 Scalene.update_line()
-        Scalene.__last_profiled_invalidated = False
-        Scalene.__last_profiled = (
-            Filename(f.f_code.co_filename),
-            LineNumber(f.f_lineno),
-            ByteCodeIndex(f.f_lasti),
-        )
+        pywhere.set_last_profiled_invalidated_false()
+        Scalene.__last_profiled[0] = Filename(f.f_code.co_filename)
+        Scalene.__last_profiled[1] = LineNumber(f.f_lineno)
+        Scalene.__last_profiled[2] = ByteCodeIndex(f.f_lasti)
+        # sys.settrace(Scalene.invalidate_lines_python)
+        # f.f_trace = Scalene.invalidate_lines_python
+        # f.f_trace_lines = True
         Scalene.__alloc_sigq.put([0])
-        # Start tracing.
-        sys.settrace(Scalene.invalidate_lines)
-        f.f_trace = Scalene.invalidate_lines
-        f.f_trace_lines = True
+        pywhere.enable_settrace()
         del this_frame
 
     @staticmethod
@@ -616,7 +716,7 @@ class Scalene:
             Scalene.__memcpy_mapfile = ScaleneMapFile("memcpy")
         except Exception:
             # Ignore if we aren't profiling memory; otherwise, exit.
-            if not arguments.cpu_only:
+            if arguments.memory:
                 sys.exit(1)
 
         Scalene.__signals.set_timer_signals(arguments.use_virtual_time)
@@ -645,8 +745,12 @@ class Scalene:
                 cmdline += " --use-virtual-time"
             if "off" in arguments and arguments.off:
                 cmdline += " --off"
-            if arguments.cpu_only:
-                cmdline += " --cpu-only"
+            if arguments.cpu:
+                cmdline += " --cpu"
+            if arguments.gpu:
+                cmdline += " --gpu"
+            if arguments.memory:
+                cmdline += " --memory"
 
             environ = ScalenePreload.get_preload_environ(arguments)
             preface = " ".join(
@@ -677,7 +781,10 @@ class Scalene:
                 + os.environ["PATH"]
             )
             # Force the executable (if anyone invokes it later) to point to one of our aliases.
-            sys.executable = Scalene.__all_python_names[0]
+            sys.executable = os.path.join(
+                Scalene.__python_alias_dir,
+                Scalene.__all_python_names[0],
+            )
 
         # Register the exit handler to run when the program terminates or we quit.
         atexit.register(Scalene.exit_handler)
@@ -792,7 +899,7 @@ class Scalene:
                 Scalene.profile_this_code,
                 Scalene.__python_alias_dir,
                 Scalene.__program_path,
-                profile_memory=not Scalene.__args.cpu_only,
+                profile_memory=Scalene.__args.memory,
             )
             if not Scalene.__output.output_file:
                 Scalene.__output.output_file = "/dev/stdout"
@@ -822,7 +929,7 @@ class Scalene:
                 Scalene.profile_this_code,
                 Scalene.__python_alias_dir,
                 Scalene.__program_path,
-                profile_memory=not Scalene.__args.cpu_only,
+                profile_memory=Scalene.__args.memory,
                 reduced_profile=Scalene.__args.reduced_profile,
             )
             return did_output
@@ -1165,6 +1272,8 @@ class Scalene:
                     )
                 )
 
+        stats.alloc_samples += len(arr)
+
         # Iterate through the array to compute the new current footprint
         # and update the global __memory_footprint_samples. Since on some systems,
         # we get free events before mallocs, force `before` to always be at least 0.
@@ -1205,21 +1314,22 @@ class Scalene:
                 ):
                     freed_last_trigger += 1
             timestamp = time.monotonic_ns() - Scalene.__start_time
-            if len(stats.memory_footprint_samples) > 2:
-                # Compress the footprints by discarding intermediate
-                # points along increases and decreases. For example:
-                # if the new point is an increase over the previous
-                # point, and that point was also an increase,
-                # eliminate the previous (intermediate) point.
-                (t1, prior_y) = stats.memory_footprint_samples[-2]
-                (t2, last_y) = stats.memory_footprint_samples[-1]
-                y = stats.current_footprint
-                if prior_y < last_y < y or prior_y > last_y > y:
-                    # Same direction.
-                    # Replace the previous (intermediate) point.
-                    stats.memory_footprint_samples[-1] = [timestamp, y]
-                else:
-                    stats.memory_footprint_samples.append([timestamp, y])
+            if False:
+                if len(stats.memory_footprint_samples) > 2:
+                    # Compress the footprints by discarding intermediate
+                    # points along increases and decreases. For example:
+                    # if the new point is an increase over the previous
+                    # point, and that point was also an increase,
+                    # eliminate the previous (intermediate) point.
+                    (t1, prior_y) = stats.memory_footprint_samples[-2]
+                    (t2, last_y) = stats.memory_footprint_samples[-1]
+                    y = stats.current_footprint
+                    if prior_y < last_y < y or prior_y > last_y > y:
+                        # Same direction.
+                        # Replace the previous (intermediate) point.
+                        stats.memory_footprint_samples[-1] = [timestamp, y]
+                    else:
+                        stats.memory_footprint_samples.append([timestamp, y])
             else:
                 stats.memory_footprint_samples.append(
                     [
@@ -1267,6 +1377,7 @@ class Scalene:
             if is_malloc and count == NEWLINE_TRIGGER_LENGTH + 1:
                 with Scalene.__invalidate_mutex:
                     last_file, last_line = Scalene.__invalidate_queue.pop(0)
+
                 stats.memory_malloc_count[last_file][last_line] += 1
                 stats.memory_aggregate_footprint[last_file][
                     last_line
@@ -1432,14 +1543,14 @@ class Scalene:
         """Return true if the filename is one we should trace."""
         if not filename:
             return False
-        if (
-            "scalene/scalene" in filename
-        ):  # lgtm [py/member-test-non-container]
+        if os.path.join("scalene", "scalene") in filename:
             # Don't profile the profiler.
             return False
+        # Don't profile the Python libraries, unless overridden by --profile-all
         if (
             "site-packages" in filename
-            or "/lib/python" in filename  # lgtm [py/member-test-non-container]
+            or f"{os.sep}lib{os.sep}python" in filename
+            or f"{os.sep}anaconda3{os.sep}lib" in filename
         ) and not Scalene.__args.profile_all:
             return False
         # Generic handling follows (when no @profile decorator has been used).
@@ -1458,7 +1569,7 @@ class Scalene:
             # create a file to hold the contents.
             import IPython
 
-            if result := re.match("ipython-input-([0-9]+)-.*", filename):
+            if result := re.match(r"ipython-input-([0-9]+)-.*", filename):
                 # Write the cell's contents into the file.
                 cell_contents = (
                     IPython.get_ipython().history_manager.input_hist_raw[
@@ -1613,6 +1724,37 @@ class Scalene:
         with contextlib.suppress(Exception):
             os.remove(f"/tmp/scalene-malloc-lock{os.getpid()}")
 
+    @staticmethod
+    def generate_html(profile_fname, output_fname):
+        """Apply a template to generate a single HTML payload containing the current profile."""
+
+        try:
+            # Load the profile
+            profile_file = pathlib.Path(profile_fname)
+            profile = profile_file.read_text()
+        except FileNotFoundError:
+            return
+
+        # Load the GUI JavaScript file.
+        scalene_dir = os.path.dirname(__file__)
+        gui_fname = os.path.join(scalene_dir, "scalene-gui", "scalene-gui.js")
+        gui_file = pathlib.Path(gui_fname)
+        gui_js = gui_file.read_text()
+
+        # Put the profile and everything else into the template.
+        environment = Environment(
+            loader=FileSystemLoader(os.path.join(scalene_dir, "scalene-gui"))
+        )
+        template = environment.get_template("index.html.template")
+        rendered_content = template.render(profile=profile, gui_js=gui_js)
+
+        # Write the rendered content to the specified output file.
+        try:
+            with open(output_fname, "w") as f:
+                f.write(rendered_content)
+        except OSError:
+            pass
+
     def profile_code(
         self,
         code: str,
@@ -1620,11 +1762,16 @@ class Scalene:
         the_locals: Dict[str, str],
     ) -> int:
         """Initiate execution and profiling."""
+        if Scalene.__args.memory:
+            from scalene import pywhere
+
+            pywhere.populate_struct()
         # If --off is set, tell all children to not profile and stop profiling before we even start.
         if "off" not in Scalene.__args or not Scalene.__args.off:
             self.start()
         # Run the code being profiled.
         exit_status = 0
+
         try:
             exec(code, the_globals, the_locals)
         except SystemExit as se:
@@ -1639,7 +1786,11 @@ class Scalene:
             exit_status = 1
         finally:
             self.stop()
-            sys.settrace(None)
+            if Scalene.__args.memory:
+                pywhere.disable_settrace()
+                pywhere.depopulate_struct()
+            # Leaving here in case of reversion
+            # sys.settrace(None)
             stats = Scalene.__stats
             (last_file, last_line, _) = Scalene.__last_profiled
             stats.memory_malloc_count[last_file][last_line] += 1
@@ -1661,84 +1812,19 @@ class Scalene:
             ):
                 return exit_status
 
-            # Start up a web server (in a background thread) to host the GUI,
-            # and open a browser tab to the server. If this fails, fail-over
-            # to using the CLI.
-
-            try:
-                PORT = Scalene.__args.port
-
-                # Silence web server output by overriding logging messages.
-                class NoLogs(http.server.SimpleHTTPRequestHandler):
-                    def log_message(
-                        self, format: str, *args: List[Any]
-                    ) -> None:
-                        return
-
-                    def log_request(
-                        self,
-                        code: Union[int, str] = 0,
-                        size: Union[int, str] = 0,
-                    ) -> None:
-                        return
-
-                Handler = NoLogs
-                socketserver.TCPServer.allow_reuse_address = True
-                with socketserver.TCPServer(("", PORT), Handler) as httpd:
-                    t = threading.Thread(target=httpd.serve_forever)
-                    # Copy files into a new directory and then point the tab there.
-                    import shutil
-
-                    webgui_dir = pathlib.Path(
-                        tempfile.mkdtemp(prefix=Scalene.__gui_dir)
-                    )
-                    shutil.copytree(
-                        os.path.join(
-                            os.path.dirname(__file__), Scalene.__gui_dir
-                        ),
-                        os.path.join(webgui_dir, Scalene.__gui_dir),
-                    )
-                    shutil.copy(
-                        Scalene.__profile_filename,
-                        os.path.join(webgui_dir, Scalene.__gui_dir),
-                    )
-                    os.chdir(os.path.join(webgui_dir, Scalene.__gui_dir))
-                    t.start()
-                    if Scalene.in_jupyter():
-                        from IPython.core.display import display
-                        from IPython.display import IFrame
-                        display(
-                            IFrame(
-                                src=f"{Scalene.__localhost}:{PORT}/{Scalene.__profiler_html}",
-                                width="100%",
-                                height=600,
-                            )
-                        )
-                    else:
-                        # Disable the preload environment
-                        # variables which are no longer needed
-                        # anyway (for memory/copy volume tracking)
-                        # and which interfere with at least one
-                        # browser on one platform.
-                        os.environ["LD_PRELOAD"] = ""
-                        os.environ["DYLD_INSERT_LIBRARIES"] = ""
-                        # Now open a tab with the profiler.
-                        webbrowser.open(
-                            f"{Scalene.__localhost}:{PORT}/{Scalene.__profiler_html}"
-                        )
-                    # Wait long enough for the server to serve the page, and then shut down the server.
-                    time.sleep(5)
-                    httpd.shutdown()
-            except OSError:
-                print(
-                    f"Scalene: unable to run the Scalene GUI on port {PORT}."
-                )
-                print("Possible solutions:")
-                print("(1) Use a different port (with --port)")
-                print("(2) Use the text version (with --cli)")
-                print(
-                    "(3) Upload a generated profile.json file to the web GUI: https://plasma-umass.org/scalene-gui/."
-                )
+            Scalene.generate_html(
+                profile_fname=Scalene.__profile_filename,
+                output_fname=Scalene.__profiler_html,
+            )
+            # Remove any interposition libraries from the environment before opening the browser.
+            # See also scalene/scalene_preload.py
+            old_dyld = os.environ.get("DYLD_INSERT_LIBRARIES", "")
+            old_ld = os.environ.get("LD_PRELOAD", "")
+            os.environ.update({"DYLD_INSERT_LIBRARIES": "", "LD_PRELOAD": ""})
+            webbrowser.open(f"file:///{os.getcwd()}/{Scalene.__profiler_html}")
+            os.environ.update(
+                {"DYLD_INSERT_LIBRARIES": old_dyld, "LD_PRELOAD": old_ld}
+            )
 
         return exit_status
 
@@ -1754,6 +1840,10 @@ class Scalene:
         Scalene.__is_child = args.pid != 0
         # the pid of the primary profiler
         Scalene.__parent_pid = args.pid if Scalene.__is_child else os.getpid()
+        # Don't profile the GPU if not enabled (i.e., either no options or --cpu and/or --memory, but no --gpu).
+        if not Scalene.__args.gpu:
+            Scalene.__output.gpu = False
+            Scalene.__json.gpu = False
 
     @staticmethod
     def set_initialized() -> None:
@@ -1826,8 +1916,25 @@ class Scalene:
             progs = None
             exit_status = 0
             try:
+                if len(sys.argv) >= 2 and sys.argv[0] == "-m":
+                    module = True
+
+                    # Remove -m and the provided module name
+                    _, mod_name, *sys.argv = sys.argv
+
+                    # Given `some.module`, find the path of the corresponding
+                    # some/module/__main__.py or some/module.py file to run.
+                    _, spec, _ = _get_module_details(mod_name)
+                    if not spec.origin:
+                        raise FileNotFoundError
+
+                    # Prepend the found .py file to arguments
+                    sys.argv.insert(0, spec.origin)
+                else:
+                    module = False
+
                 # Look for something ending in '.py'. Treat the first one as our executable.
-                progs = [x for x in sys.argv if re.match(".*\.py$", x)]
+                progs = [x for x in sys.argv if re.match(r".*\.py$", x)]
                 # Just in case that didn't work, try sys.argv[0] and __file__.
                 with contextlib.suppress(Exception):
                     progs.extend((sys.argv[0], __file__))
@@ -1847,7 +1954,9 @@ class Scalene:
                         sys.exit(1)
                     # Push the program's path.
                     program_path = os.path.dirname(os.path.abspath(progs[0]))
-                    sys.path.insert(0, program_path)
+                    if not module:
+                        sys.path.insert(0, program_path)
+
                     # If a program path was specified at the command-line, use it.
                     if len(args.program_path) > 0:
                         Scalene.__program_path = os.path.abspath(
@@ -1857,7 +1966,7 @@ class Scalene:
                         # Otherwise, use the invoked directory.
                         Scalene.__program_path = os.getcwd()
                     # Grab local and global variables.
-                    if not Scalene.__args.cpu_only:
+                    if Scalene.__args.memory:
                         from scalene import pywhere  # type: ignore
 
                         pywhere.register_files_to_profile(
@@ -1865,7 +1974,6 @@ class Scalene:
                             Scalene.__program_path,
                             Scalene.__args.profile_all,
                         )
-
                     import __main__
 
                     the_locals = __main__.__dict__
