@@ -25,11 +25,12 @@
 
 #include "common.hpp"
 #include "mallocrecursionguard.hpp"
+#include "poissonsampler.hpp"
 #include "printf.h"
 #include "pywhere.hpp"
 #include "samplefile.hpp"
-#include "sampleinterval.hpp"
 #include "scaleneheader.hpp"
+#include "thresholdsampler.hpp"
 
 static SampleFile& getSampleFile() {
   static SampleFile mallocSampleFile("/tmp/scalene-malloc-signal%d",
@@ -147,7 +148,6 @@ class SampleHeap : public SuperHeap {
                               bool inPythonAllocator = true) {
     assert(realSize);
     // If this is the special NEWLINE value, trigger an update.
-    // printf("uwu %p\n", &in_realloc);
     if (unlikely(realSize == NEWLINE)) {
       std::string filename;
       int lineno;
@@ -159,14 +159,16 @@ class SampleHeap : public SuperHeap {
       mallocTriggered()++;
       return;
     }
-    auto sampleMalloc = _allocationSampler.increment(realSize);
+    size_t sampleMallocSize;
+    auto sampleMalloc =
+        _allocationSampler.increment(realSize, ptr, sampleMallocSize);
     if (inPythonAllocator) {
       _pythonCount += realSize;
     } else {
       _cCount += realSize;
     }
     if (unlikely(sampleMalloc)) {
-      process_malloc(sampleMalloc, ptr);
+      process_malloc(sampleMallocSize, ptr);
     }
   }
 
@@ -204,13 +206,15 @@ class SampleHeap : public SuperHeap {
   }
 
   inline void register_free(size_t realSize, void* ptr) {
-    auto sampleFree = _allocationSampler.decrement(realSize);
+    size_t sampleFreeSize;
+    auto sampleFree =
+        _allocationSampler.decrement(realSize, ptr, sampleFreeSize);
 
     if (unlikely(ptr && (ptr == _lastMallocTrigger))) {
       _freedLastMallocTrigger = true;
     }
     if (unlikely(sampleFree)) {
-      process_free(sampleFree);
+      process_free(sampleFreeSize);
     }
   }
 
@@ -269,7 +273,14 @@ class SampleHeap : public SuperHeap {
 
   void* _lastMallocTrigger;
   bool _freedLastMallocTrigger;
-  SampleInterval _allocationSampler;
+#if 0
+  typedef PoissonSampler Sampler;
+#warning "Experimental use only: Poisson sampler"
+#else
+  typedef ThresholdSampler Sampler;
+#endif
+
+  Sampler _allocationSampler;
 
   static constexpr auto flags = O_RDWR | O_CREAT;
   static constexpr auto perms = S_IRUSR | S_IWUSR;
