@@ -1,15 +1,11 @@
 import numpy as np
-import os
 #import math
 
 from numpy import linalg as LA
 
 arr = [i for i in range(1,1000)]
 
-import time
-
 def doit1(x):
-    time.sleep(0.1)
 #    x = [i*i for i in range(1,1000)][0]
     y = 1
 #    w, v = LA.eig(np.diag(arr)) # (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)))
@@ -21,7 +17,6 @@ def doit1(x):
     return z
 
 def doit2(x):
-    time.sleep(0.2)
     i = 0
 #    zarr = [math.cos(13) for i in range(1,100000)]
 #    z = zarr[0]
@@ -50,7 +45,7 @@ def doit3(x):
 def stuff():
     y = np.random.randint(1, 100, size=5000000)[4999999]
     x = 1.01
-    for i in range(1,3):
+    for i in range(1,10):
         print(i)
         for j in range(1,10):
             x = doit1(x)
